@@ -21,6 +21,7 @@
 
 #include <KDirLister>
 #include <KDirModel>
+#include <KGlobalSettings>
 #include <KLocale>
 #include <KStringHandler>
 #include <KDebug>
@@ -33,7 +34,7 @@
 KFileItemModel::KFileItemModel(KDirLister* dirLister, QObject* parent) :
     KItemModelBase("name", parent),
     m_dirLister(dirLister),
-    m_naturalSorting(true),
+    m_naturalSorting(KGlobalSettings::naturalSorting()),
     m_sortFoldersFirst(true),
     m_sortRole(NameRole),
     m_roles(),
@@ -49,7 +50,7 @@ KFileItemModel::KFileItemModel(KDirLister* dirLister, QObject* parent) :
     m_pendingItemsToInsert(),
     m_pendingEmitLoadingCompleted(false),
     m_groups(),
-    m_rootExpansionLevel(UninitializedRootExpansionLevel),
+    m_expandedParentsCountRoot(UninitializedExpandedParentsCountRoot),
     m_expandedUrls(),
     m_urlsToExpand()
 {
@@ -84,7 +85,7 @@ KFileItemModel::KFileItemModel(KDirLister* dirLister, QObject* parent) :
     m_maximumUpdateIntervalTimer->setInterval(2000);
     m_maximumUpdateIntervalTimer->setSingleShot(true);
     connect(m_maximumUpdateIntervalTimer, SIGNAL(timeout()), this, SLOT(dispatchPendingItemsToInsert()));
-    
+
     // When changing the value of an item which represents the sort-role a resorting must be
     // triggered. Especially in combination with KFileItemModelRolesUpdater this might be done
     // for a lot of items within a quite small timeslot. To prevent expensive resortings the
@@ -95,6 +96,8 @@ KFileItemModel::KFileItemModel(KDirLister* dirLister, QObject* parent) :
     connect(m_resortAllItemsTimer, SIGNAL(timeout()), this, SLOT(resortAllItems()));
 
     Q_ASSERT(m_minimumUpdateIntervalTimer->interval() <= m_maximumUpdateIntervalTimer->interval());
+
+    connect(KGlobalSettings::self(), SIGNAL(naturalSortingChanged()), this, SLOT(slotNaturalSortingChanged()));
 }
 
 KFileItemModel::~KFileItemModel()
@@ -148,7 +151,7 @@ bool KFileItemModel::setData(int index, const QHash<QByteArray, QVariant>& value
     if (changedRoles.contains(sortRole())) {
         m_resortAllItemsTimer->start();
     }
-        
+
     return true;
 }
 
@@ -254,7 +257,7 @@ int KFileItemModel::indexForKeyboardSearch(const QString& text, int startFromInd
 bool KFileItemModel::supportsDropping(int index) const
 {
     const KFileItem item = fileItem(index);
-    return item.isNull() ? false : item.isDir();
+    return item.isNull() ? false : item.isDir() || item.isDesktopFile();
 }
 
 QString KFileItemModel::roleDescription(const QByteArray& role) const
@@ -262,22 +265,22 @@ QString KFileItemModel::roleDescription(const QByteArray& role) const
     QString descr;
 
     switch (roleIndex(role)) {
-    case NameRole:           descr = i18nc("@item:intable", "Name"); break;
-    case SizeRole:           descr = i18nc("@item:intable", "Size"); break;
-    case DateRole:           descr = i18nc("@item:intable", "Date"); break;
-    case PermissionsRole:    descr = i18nc("@item:intable", "Permissions"); break;
-    case OwnerRole:          descr = i18nc("@item:intable", "Owner"); break;
-    case GroupRole:          descr = i18nc("@item:intable", "Group"); break;
-    case TypeRole:           descr = i18nc("@item:intable", "Type"); break;
-    case DestinationRole:    descr = i18nc("@item:intable", "Destination"); break;
-    case PathRole:           descr = i18nc("@item:intable", "Path"); break;
-    case CommentRole:        descr = i18nc("@item:intable", "Comment"); break;
-    case TagsRole:           descr = i18nc("@item:intable", "Tags"); break;
-    case RatingRole:         descr = i18nc("@item:intable", "Rating"); break;
-    case NoRole:             break;
-    case IsDirRole:          break;
-    case IsExpandedRole:     break;
-    case ExpansionLevelRole: break;
+    case NameRole:        descr = i18nc("@item:intable", "Name"); break;
+    case SizeRole:        descr = i18nc("@item:intable", "Size"); break;
+    case DateRole:        descr = i18nc("@item:intable", "Date"); break;
+    case PermissionsRole: descr = i18nc("@item:intable", "Permissions"); break;
+    case OwnerRole:       descr = i18nc("@item:intable", "Owner"); break;
+    case GroupRole:       descr = i18nc("@item:intable", "Group"); break;
+    case TypeRole:        descr = i18nc("@item:intable", "Type"); break;
+    case DestinationRole: descr = i18nc("@item:intable", "Destination"); break;
+    case PathRole:        descr = i18nc("@item:intable", "Path"); break;
+    case CommentRole:     descr = i18nc("@item:intable", "Comment"); break;
+    case TagsRole:        descr = i18nc("@item:intable", "Tags"); break;
+    case RatingRole:      descr = i18nc("@item:intable", "Rating"); break;
+    case NoRole:          break;
+    case IsDirRole:       break;
+    case IsExpandedRole:  break;
+    case ExpandedParentsCountRole: break;
     default:                 Q_ASSERT(false); break;
     }
 
@@ -292,22 +295,22 @@ QList<QPair<int, QVariant> > KFileItemModel::groups() const
         timer.start();
 #endif
         switch (roleIndex(sortRole())) {
-        case NameRole:           m_groups = nameRoleGroups(); break;
-        case SizeRole:           m_groups = sizeRoleGroups(); break;
-        case DateRole:           m_groups = dateRoleGroups(); break;
-        case PermissionsRole:    m_groups = permissionRoleGroups(); break;
-        case OwnerRole:          m_groups = genericStringRoleGroups("owner"); break;
-        case GroupRole:          m_groups = genericStringRoleGroups("group"); break;
-        case TypeRole:           m_groups = genericStringRoleGroups("type"); break;
-        case DestinationRole:    m_groups = genericStringRoleGroups("destination"); break;
-        case PathRole:           m_groups = genericStringRoleGroups("path"); break;
-        case CommentRole:        m_groups = genericStringRoleGroups("comment"); break;
-        case TagsRole:           m_groups = genericStringRoleGroups("tags"); break;
-        case RatingRole:         m_groups = ratingRoleGroups(); break;
-        case NoRole:             break;
-        case IsDirRole:          break;
-        case IsExpandedRole:     break;
-        case ExpansionLevelRole: break;
+        case NameRole:        m_groups = nameRoleGroups(); break;
+        case SizeRole:        m_groups = sizeRoleGroups(); break;
+        case DateRole:        m_groups = dateRoleGroups(); break;
+        case PermissionsRole: m_groups = permissionRoleGroups(); break;
+        case OwnerRole:       m_groups = genericStringRoleGroups("owner"); break;
+        case GroupRole:       m_groups = genericStringRoleGroups("group"); break;
+        case TypeRole:        m_groups = genericStringRoleGroups("type"); break;
+        case DestinationRole: m_groups = genericStringRoleGroups("destination"); break;
+        case PathRole:        m_groups = genericStringRoleGroups("path"); break;
+        case CommentRole:     m_groups = genericStringRoleGroups("comment"); break;
+        case TagsRole:        m_groups = genericStringRoleGroups("tags"); break;
+        case RatingRole:      m_groups = ratingRoleGroups(); break;
+        case NoRole:          break;
+        case IsDirRole:       break;
+        case IsExpandedRole:  break;
+        case ExpandedParentsCountRole: break;
         default:                 Q_ASSERT(false); break;
         }
 
@@ -369,11 +372,14 @@ void KFileItemModel::clear()
 
 void KFileItemModel::setRoles(const QSet<QByteArray>& roles)
 {
+    if (m_roles == roles) {
+        return;
+    }
     m_roles = roles;
 
     if (count() > 0) {
-        const bool supportedExpanding = m_requestRole[ExpansionLevelRole];
-        const bool willSupportExpanding = roles.contains("expansionLevel");
+        const bool supportedExpanding = m_requestRole[ExpandedParentsCountRole];
+        const bool willSupportExpanding = roles.contains("expandedParentsCount");
         if (supportedExpanding && !willSupportExpanding) {
             // No expanding is supported anymore. Take care to delete all items that have an expansion level
             // that is not 0 (and hence are part of an expanded item).
@@ -436,9 +442,9 @@ bool KFileItemModel::setExpanded(int index, bool expanded)
         }
 
         KFileItemList itemsToRemove;
-        const int expansionLevel = data(index)["expansionLevel"].toInt();
+        const int expandedParentsCount = data(index)["expandedParentsCount"].toInt();
         ++index;
-        while (index < count() && data(index)["expansionLevel"].toInt() > expansionLevel) {
+        while (index < count() && data(index)["expandedParentsCount"].toInt() > expandedParentsCount) {
             itemsToRemove.append(m_itemData.at(index)->item);
             ++index;
         }
@@ -465,6 +471,17 @@ bool KFileItemModel::isExpandable(int index) const
     return false;
 }
 
+int KFileItemModel::expandedParentsCount(int index) const
+{
+    if (index >= 0 && index < count()) {
+        const int parentsCount = m_itemData.at(index)->values.value("expandedParentsCount").toInt();
+        if (parentsCount > 0) {
+            return parentsCount;
+        }
+    }
+    return 0;
+}
+
 QSet<KUrl> KFileItemModel::expandedUrls() const
 {
     return m_expandedUrls;
@@ -475,29 +492,24 @@ void KFileItemModel::restoreExpandedUrls(const QSet<KUrl>& urls)
     m_urlsToExpand = urls;
 }
 
-void KFileItemModel::setExpanded(const QSet<KUrl>& urls)
+void KFileItemModel::expandParentItems(const KUrl& url)
 {
     const KDirLister* dirLister = m_dirLister.data();
     if (!dirLister) {
         return;
     }
 
-    const int pos = dirLister->url().url().length();
+    const int pos = dirLister->url().path().length();
 
-    // Assure that each sub-path of the URLs that should be
-    // expanded is added to m_urlsToExpand too. KDirLister
+    // Assure that each sub-path of the URL that should be
+    // expanded is added to m_urlsToExpand. KDirLister
     // does not care whether the parent-URL has already been
     // expanded.
-    QSetIterator<KUrl> it1(urls);
-    while (it1.hasNext()) {
-        const KUrl& url = it1.next();
-
-        KUrl urlToExpand = dirLister->url();
-        const QStringList subDirs = url.url().mid(pos).split(QDir::separator());
-        for (int i = 0; i < subDirs.count(); ++i) {
-            urlToExpand.addPath(subDirs.at(i));
-            m_urlsToExpand.insert(urlToExpand);
-        }
+    KUrl urlToExpand = dirLister->url();
+    const QStringList subDirs = url.path().mid(pos).split(QDir::separator());
+    for (int i = 0; i < subDirs.count() - 1; ++i) {
+        urlToExpand.addPath(subDirs.at(i));
+        m_urlsToExpand.insert(urlToExpand);
     }
 
     // KDirLister::open() must called at least once to trigger an initial
@@ -583,13 +595,13 @@ void KFileItemModel::onSortOrderChanged(Qt::SortOrder current, Qt::SortOrder pre
 {
     Q_UNUSED(current);
     Q_UNUSED(previous);
-    resortAllItems();    
+    resortAllItems();
 }
 
 void KFileItemModel::resortAllItems()
 {
     m_resortAllItemsTimer->stop();
-    
+
     const int itemCount = count();
     if (itemCount <= 0) {
         return;
@@ -610,23 +622,23 @@ void KFileItemModel::resortAllItems()
     foreach (const ItemData* itemData, m_itemData) {
         oldUrls.append(itemData->item.url());
     }
-   
+
     m_groups.clear();
     m_items.clear();
-    
+
     // Resort the items
-    sort(m_itemData.begin(), m_itemData.end());    
+    sort(m_itemData.begin(), m_itemData.end());
     for (int i = 0; i < itemCount; ++i) {
         m_items.insert(m_itemData.at(i)->item.url(), i);
     }
-    
+
     // Determine the indexes that have been moved
     QList<int> movedToIndexes;
     movedToIndexes.reserve(itemCount);
     for (int i = 0; i < itemCount; i++) {
         const int newIndex = m_items.value(oldUrls.at(i).url());
         movedToIndexes.append(newIndex);
-    }   
+    }
 
     // Don't check whether items have really been moved and always emit a
     // itemsMoved() signal after resorting: In case of grouped items
@@ -634,10 +646,10 @@ void KFileItemModel::resortAllItems()
     // position. Let the receiver of the signal decide whether a check for moved
     // items makes sense.
     emit itemsMoved(KItemRange(0, itemCount), movedToIndexes);
-    
+
 #ifdef KFILEITEMMODEL_DEBUG
     kDebug() << "[TIME] Resorting of" << itemCount << "items:" << timer.elapsed();
-#endif    
+#endif
 }
 
 void KFileItemModel::slotCompleted()
@@ -689,7 +701,7 @@ void KFileItemModel::slotNewItems(const KFileItemList& items)
 {
     Q_ASSERT(!items.isEmpty());
 
-    if (m_requestRole[ExpansionLevelRole] && m_rootExpansionLevel >= 0) {
+    if (m_requestRole[ExpandedParentsCountRole] && m_expandedParentsCountRoot >= 0) {
         // To be able to compare whether the new items may be inserted as children
         // of a parent item the pending items must be added to the model first.
         dispatchPendingItemsToInsert();
@@ -749,7 +761,7 @@ void KFileItemModel::slotItemsDeleted(const KFileItemList& items)
     dispatchPendingItemsToInsert();
 
     KFileItemList itemsToRemove = items;
-    if (m_requestRole[ExpansionLevelRole] && m_rootExpansionLevel >= 0) {
+    if (m_requestRole[ExpandedParentsCountRole] && m_expandedParentsCountRoot >= 0) {
         // Assure that removing a parent item also results in removing all children
         foreach (const KFileItem& item, items) {
             itemsToRemove.append(childItems(item));
@@ -844,7 +856,7 @@ void KFileItemModel::slotClear()
     m_resortAllItemsTimer->stop();
     m_pendingItemsToInsert.clear();
 
-    m_rootExpansionLevel = UninitializedRootExpansionLevel;
+    m_expandedParentsCountRoot = UninitializedExpandedParentsCountRoot;
 
     const int removedCount = m_itemData.count();
     if (removedCount > 0) {
@@ -862,6 +874,12 @@ void KFileItemModel::slotClear(const KUrl& url)
     Q_UNUSED(url);
 }
 
+void KFileItemModel::slotNaturalSortingChanged()
+{
+    m_naturalSorting = KGlobalSettings::naturalSorting();
+    resortAllItems();
+}
+
 void KFileItemModel::dispatchPendingItemsToInsert()
 {
     if (!m_pendingItemsToInsert.isEmpty()) {
@@ -923,7 +941,7 @@ void KFileItemModel::insertItems(const KFileItemList& items)
         // Insert item at the position targetIndex by transfering
         // the ownership of the item-data from sortedItems to m_itemData.
         // m_items will be inserted after the loop (see comment below)
-        m_itemData.insert(targetIndex, sortedItems.at(sourceIndex));        
+        m_itemData.insert(targetIndex, sortedItems.at(sourceIndex));
         ++insertedCount;
 
         if (insertedAtIndex < 0) {
@@ -981,7 +999,7 @@ void KFileItemModel::removeItems(const KFileItemList& items)
     int targetIndex = 0;
     foreach (const ItemData* itemData, sortedItems) {
         const KFileItem& itemToRemove = itemData->item;
-        
+
         const int previousTargetIndex = targetIndex;
         while (targetIndex < m_itemData.count()) {
             if (m_itemData.at(targetIndex)->item.url() == itemToRemove.url()) {
@@ -1027,7 +1045,7 @@ void KFileItemModel::removeItems(const KFileItemList& items)
     }
 
     if (count() <= 0) {
-        m_rootExpansionLevel = UninitializedRootExpansionLevel;
+        m_expandedParentsCountRoot = UninitializedExpandedParentsCountRoot;
     }
 
     itemRanges << KItemRange(removedAtIndex, removedCount);
@@ -1045,8 +1063,8 @@ QList<KFileItemModel::ItemData*> KFileItemModel::createItemDataList(const KFileI
         itemData->values = retrieveData(item);
         itemData->parent = 0;
 
-        const bool determineParent = m_requestRole[ExpansionLevelRole]
-                                     && itemData->values["expansionLevel"].toInt() > 0;
+        const bool determineParent = m_requestRole[ExpandedParentsCountRole]
+                                     && itemData->values["expandedParentsCount"].toInt() > 0;
         if (determineParent) {
             KUrl parentUrl = item.url().upUrl();
             parentUrl.adjustPath(KUrl::RemoveTrailingSlash);
@@ -1060,7 +1078,7 @@ QList<KFileItemModel::ItemData*> KFileItemModel::createItemDataList(const KFileI
 
         itemDataList.append(itemData);
     }
- 
+
     return itemDataList;
 }
 
@@ -1071,17 +1089,17 @@ void KFileItemModel::removeExpandedItems()
     const int maxIndex = m_itemData.count() - 1;
     for (int i = 0; i <= maxIndex; ++i) {
         const ItemData* itemData = m_itemData.at(i);
-        if (itemData->values.value("expansionLevel").toInt() > 0) {
+        if (itemData->values.value("expandedParentsCount").toInt() > 0) {
             expandedItems.append(itemData->item);
         }
     }
 
-    // The m_rootExpansionLevel may not get reset before all items with
-    // a bigger expansionLevel have been removed.
-    Q_ASSERT(m_rootExpansionLevel >= 0);
+    // The m_expandedParentsCountRoot may not get reset before all items with
+    // a bigger count have been removed.
+    Q_ASSERT(m_expandedParentsCountRoot >= 0);
     removeItems(expandedItems);
 
-    m_rootExpansionLevel = UninitializedRootExpansionLevel;
+    m_expandedParentsCountRoot = UninitializedExpandedParentsCountRoot;
     m_expandedUrls.clear();
 }
 
@@ -1111,13 +1129,37 @@ KFileItemModel::Role KFileItemModel::roleIndex(const QByteArray& role) const
         rolesHash.insert("isDir", IsDirRole);
         rolesHash.insert("isExpanded", IsExpandedRole);
         rolesHash.insert("isExpandable", IsExpandableRole);
-        rolesHash.insert("expansionLevel", ExpansionLevelRole);
+        rolesHash.insert("expandedParentsCount", ExpandedParentsCountRole);
     }
     return rolesHash.value(role, NoRole);
 }
 
+QByteArray KFileItemModel::roleByteArray(Role role) const
+{
+    static const char* const roles[RolesCount] = {
+        0, // NoRole
+        "name",
+        "size",
+        "date",
+        "permissions",
+        "owner",
+        "group",
+        "type",
+        "destination",
+        "path",
+        "comment",
+        "tags",
+        "rating",
+        "isDir",
+        "isExpanded",
+        "isExpandable",
+        "expandedParentsCount"
+    };
+    return roles[role];
+}
+
 QHash<QByteArray, QVariant> KFileItemModel::retrieveData(const KFileItem& item) const
-{    
+{
     // It is important to insert only roles that are fast to retrieve. E.g.
     // KFileItem::iconName() can be very expensive if the MIME-type is unknown
     // and hence will be retrieved asynchronously by KFileItemModelRolesUpdater.
@@ -1191,28 +1233,28 @@ QHash<QByteArray, QVariant> KFileItemModel::retrieveData(const KFileItem& item)
         data.insert("isExpandable", item.isDir() && item.url() == item.targetUrl());
     }
 
-    if (m_requestRole[ExpansionLevelRole]) {
-        if (m_rootExpansionLevel == UninitializedRootExpansionLevel && m_dirLister.data()) {
+    if (m_requestRole[ExpandedParentsCountRole]) {
+        if (m_expandedParentsCountRoot == UninitializedExpandedParentsCountRoot && m_dirLister.data()) {
             const KUrl rootUrl = m_dirLister.data()->url();
             const QString protocol = rootUrl.protocol();
-            const bool forceRootExpansionLevel = (protocol == QLatin1String("trash") ||
-                                                  protocol == QLatin1String("nepomuk") ||
-                                                  protocol == QLatin1String("remote") ||
-                                                  protocol.contains(QLatin1String("search")));
-            if (forceRootExpansionLevel) {
-                m_rootExpansionLevel = ForceRootExpansionLevel;
+            const bool forceExpandedParentsCountRoot = (protocol == QLatin1String("trash") ||
+                                                        protocol == QLatin1String("nepomuk") ||
+                                                        protocol == QLatin1String("remote") ||
+                                                        protocol.contains(QLatin1String("search")));
+            if (forceExpandedParentsCountRoot) {
+                m_expandedParentsCountRoot = ForceExpandedParentsCountRoot;
             } else {
                 const QString rootDir = rootUrl.path(KUrl::AddTrailingSlash);
-                m_rootExpansionLevel = rootDir.count('/');
+                m_expandedParentsCountRoot = rootDir.count('/');
             }
         }
 
-        if (m_rootExpansionLevel == ForceRootExpansionLevel) {
-            data.insert("expansionLevel", -1);
+        if (m_expandedParentsCountRoot == ForceExpandedParentsCountRoot) {
+            data.insert("expandedParentsCount", -1);
         } else {
             const QString dir = item.url().directory(KUrl::AppendTrailingSlash);
-            const int level = dir.count('/') - m_rootExpansionLevel;
-            data.insert("expansionLevel", level);
+            const int level = dir.count('/') - m_expandedParentsCountRoot;
+            data.insert("expandedParentsCount", level);
         }
     }
 
@@ -1231,8 +1273,8 @@ bool KFileItemModel::lessThan(const ItemData* a, const ItemData* b) const
 {
     int result = 0;
 
-    if (m_rootExpansionLevel >= 0) {
-        result = expansionLevelsCompare(a, b);
+    if (m_expandedParentsCountRoot >= 0) {
+        result = expandedParentsCountCompare(a, b);
         if (result != 0) {
             // The items have parents with different expansion levels
             return (sortOrder() == Qt::AscendingOrder) ? result < 0 : result > 0;
@@ -1266,6 +1308,38 @@ int KFileItemModel::sortRoleCompare(const ItemData* a, const ItemData* b) const
         // The name role is handled as default fallback after the switch
         break;
 
+    case SizeRole: {
+        if (itemA.isDir()) {
+            // See "if (m_sortFoldersFirst || m_sortRole == SizeRole)" in KFileItemModel::lessThan():
+            Q_ASSERT(itemB.isDir());
+
+            const QVariant valueA = a->values.value("size");
+            const QVariant valueB = b->values.value("size");
+            if (valueA.isNull() && valueB.isNull()) {
+                result = 0;
+            } else if (valueA.isNull()) {
+                result = -1;
+            } else if (valueB.isNull()) {
+                result = +1;
+            } else {
+                result = valueA.toInt() - valueB.toInt();
+            }
+        } else {
+            // See "if (m_sortFoldersFirst || m_sortRole == SizeRole)" in KFileItemModel::lessThan():
+            Q_ASSERT(!itemB.isDir());
+            const KIO::filesize_t sizeA = itemA.size();
+            const KIO::filesize_t sizeB = itemB.size();
+            if (sizeA > sizeB) {
+                result = +1;
+            } else if (sizeA < sizeB) {
+                result = -1;
+            } else {
+                result = 0;
+            }
+        }
+        break;
+    }
+
     case DateRole: {
         const KDateTime dateTimeA = itemA.time(KFileItem::ModificationTime);
         const KDateTime dateTimeB = itemB.time(KFileItem::ModificationTime);
@@ -1277,50 +1351,25 @@ int KFileItemModel::sortRoleCompare(const ItemData* a, const ItemData* b) const
         break;
     }
 
-    case SizeRole: {
-        if (itemA.isDir()) {
-            Q_ASSERT(itemB.isDir()); // see "if (m_sortFoldersFirst || m_sortRole == SizeRole)" above
-
-            const QVariant valueA = a->values.value("size");
-            const QVariant valueB = b->values.value("size");
-
-            if (valueA.isNull()) {
-                result = -1;
-            } else if (valueB.isNull()) {
-                result = +1;
-            } else {
-                result = valueA.value<KIO::filesize_t>() - valueB.value<KIO::filesize_t>();
-            }
-        } else {
-            Q_ASSERT(!itemB.isDir()); // see "if (m_sortFoldersFirst || m_sortRole == SizeRole)" above
-            result = itemA.size() - itemB.size();
-        }
-        break;
-    }
-
-    case TypeRole: {
-        result = QString::compare(a->values.value("type").toString(),
-                                  b->values.value("type").toString());
-        break;
-    }
-
-    case CommentRole: {
-        result = QString::compare(a->values.value("comment").toString(),
-                                  b->values.value("comment").toString());
-        break;
-    }
-
-    case TagsRole: {
-        result = QString::compare(a->values.value("tags").toString(),
-                                  b->values.value("tags").toString());
-        break;
-    }
-
     case RatingRole: {
         result = a->values.value("rating").toInt() - b->values.value("rating").toInt();
         break;
     }
 
+    case PermissionsRole:
+    case OwnerRole:
+    case GroupRole:
+    case TypeRole:
+    case DestinationRole:
+    case PathRole:
+    case CommentRole:
+    case TagsRole: {
+        const QByteArray role = roleByteArray(m_sortRole);
+        result = QString::compare(a->values.value(role).toString(),
+                                  b->values.value(role).toString());
+        break;
+    }
+
     default:
         break;
     }
@@ -1351,16 +1400,16 @@ int KFileItemModel::sortRoleCompare(const ItemData* a, const ItemData* b) const
 
 void KFileItemModel::sort(QList<ItemData*>::iterator begin,
                           QList<ItemData*>::iterator end)
-{   
+{
     // The implementation is based on qStableSortHelper() from qalgorithms.h
     // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
     // In opposite to qStableSort() it allows to use a member-function for the comparison of elements.
-    
+
     const int span = end - begin;
     if (span < 2) {
         return;
     }
-    
+
     const QList<ItemData*>::iterator middle = begin + span / 2;
     sort(begin, middle);
     sort(middle, end);
@@ -1373,21 +1422,21 @@ void KFileItemModel::merge(QList<ItemData*>::iterator begin,
 {
     // The implementation is based on qMerge() from qalgorithms.h
     // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-    
+
     const int len1 = pivot - begin;
     const int len2 = end - pivot;
-    
+
     if (len1 == 0 || len2 == 0) {
         return;
     }
-    
+
     if (len1 + len2 == 2) {
         if (lessThan(*(begin + 1), *(begin))) {
             qSwap(*begin, *(begin + 1));
         }
         return;
     }
-    
+
     QList<ItemData*>::iterator firstCut;
     QList<ItemData*>::iterator secondCut;
     int len2Half;
@@ -1401,11 +1450,11 @@ void KFileItemModel::merge(QList<ItemData*>::iterator begin,
         secondCut = pivot + len2Half;
         firstCut = upperBound(begin, pivot, *secondCut);
     }
-    
+
     reverse(firstCut, pivot);
     reverse(pivot, secondCut);
     reverse(firstCut, secondCut);
-    
+
     const QList<ItemData*>::iterator newPivot = firstCut + len2Half;
     merge(begin, firstCut, newPivot);
     merge(newPivot, secondCut, end);
@@ -1417,7 +1466,7 @@ QList<KFileItemModel::ItemData*>::iterator KFileItemModel::lowerBound(QList<Item
 {
     // The implementation is based on qLowerBound() from qalgorithms.h
     // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-    
+
     QList<ItemData*>::iterator middle;
     int n = int(end - begin);
     int half;
@@ -1441,7 +1490,7 @@ QList<KFileItemModel::ItemData*>::iterator KFileItemModel::upperBound(QList<Item
 {
     // The implementation is based on qUpperBound() from qalgorithms.h
     // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-    
+
     QList<ItemData*>::iterator middle;
     int n = end - begin;
     int half;
@@ -1464,11 +1513,11 @@ void KFileItemModel::reverse(QList<ItemData*>::iterator begin,
 {
     // The implementation is based on qReverse() from qalgorithms.h
     // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-    
+
     --end;
     while (begin < end) {
         qSwap(*begin++, *end--);
-    }    
+    }
 }
 
 int KFileItemModel::stringCompare(const QString& a, const QString& b) const
@@ -1493,7 +1542,7 @@ int KFileItemModel::stringCompare(const QString& a, const QString& b) const
                             : QString::compare(a, b, Qt::CaseSensitive);
 }
 
-int KFileItemModel::expansionLevelsCompare(const ItemData* a, const ItemData* b) const
+int KFileItemModel::expandedParentsCountCompare(const ItemData* a, const ItemData* b) const
 {
     const KUrl urlA = a->item.url();
     const KUrl urlB = b->item.url();
@@ -1900,9 +1949,9 @@ KFileItemList KFileItemModel::childItems(const KFileItem& item) const
 
     int index = m_items.value(item.url(), -1);
     if (index >= 0) {
-        const int parentLevel = m_itemData.at(index)->values.value("expansionLevel").toInt();
+        const int parentLevel = m_itemData.at(index)->values.value("expandedParentsCount").toInt();
         ++index;
-        while (index < m_itemData.count() && m_itemData.at(index)->values.value("expansionLevel").toInt() > parentLevel) {
+        while (index < m_itemData.count() && m_itemData.at(index)->values.value("expandedParentsCount").toInt() > parentLevel) {
             items.append(m_itemData.at(index)->item);
             ++index;
         }
