@@ -72,18 +72,33 @@ public:
     KItemListView(QGraphicsWidget* parent = 0);
     virtual ~KItemListView();
 
+    /**
+     * If the scroll-orientation is vertical, the items are ordered
+     * from top to bottom (= default setting). If the scroll-orientation
+     * is horizontal, the items are ordered from left to right.
+     */
     void setScrollOrientation(Qt::Orientation orientation);
     Qt::Orientation scrollOrientation() const;
 
     void setItemSize(const QSizeF& size);
     QSizeF itemSize() const;
 
-    // TODO: add note that offset is not checked against maximumOffset, only against 0.
+    /**
+     * Offset of the scrollbar that represents the scroll-orientation
+     * (see setScrollOrientation()).
+     */
     void setScrollOffset(qreal offset);
     qreal scrollOffset() const;
 
     qreal maximumScrollOffset() const;
 
+    /**
+     * Offset related to an item, that does not fit into the available
+     * size of the listview. If the scroll-orientation is vertical
+     * the item-offset describes the offset of the horizontal axe, if
+     * the scroll-orientation is horizontal the item-offset describes
+     * the offset of the vertical axe.
+     */
     void setItemOffset(qreal scrollOffset);
     qreal itemOffset() const;
 
@@ -116,7 +131,7 @@ public:
 
     /**
      * @return Model of the item-list. The model gets
-     *         initialized by KItemListController::setView() and will
+     *         initialized by KItemListController::setModel() and will
      *         result in calling KItemListController::onModelChanged().
      */
     KItemModelBase* model() const;
@@ -145,11 +160,27 @@ public:
     /** @reimp */
     virtual void setGeometry(const QRectF& rect);
 
+    /**
+     * @return Index of the item that is below the point \a pos.
+     *         The position is relative to the upper right of
+     *         the visible area. Only (at least partly) visible
+     *         items are considered. -1 is returned if no item is
+     *         below the position.
+     */
     int itemAt(const QPointF& pos) const;
     bool isAboveSelectionToggle(int index, const QPointF& pos) const;
     bool isAboveExpansionToggle(int index, const QPointF& pos) const;
 
+    /**
+     * @return Index of the first item that is at least partly visible.
+     *         -1 is returned if the model contains no items.
+     */
     int firstVisibleIndex() const;
+
+    /**
+     * @return Index of the last item that is at least partly visible.
+     *         -1 is returned if the model contains no items.
+     */
     int lastVisibleIndex() const;
 
     /**
@@ -162,21 +193,24 @@ public:
 
     /**
      * @param itemRanges Items that must be checked for getting the visible roles sizes.
-     * @return The size of each visible role in case if KItemListView::itemSize()
-     *         is empty. This allows to have dynamic but equal role sizes between
-     *         all items. Per default an empty hash is returned.
+     * @return           The size of each visible role in case if KItemListView::itemSize()
+     *                   is empty. This allows to have dynamic but equal role sizes between
+     *                   all items, like used in the classic "table-views". Per default an
+     *                   empty hash is returned.
      */
     virtual QHash<QByteArray, QSizeF> visibleRolesSizes(const KItemRangeList& itemRanges) const;
 
     /**
-     * @return True if the view supports the expanding of items. Per default false
-     *         is returned. If expanding of items is supported, the methods
-     *         KItemModelBase::setExpanded(), KItemModelBase::isExpanded() and
-     *         KItemModelBase::isExpandable() must be reimplemented. The view-implementation
+     * @return True if the view supports the expanding of items. Per default
+     *         false is returned. If expanding of items is supported
+     *         (see setSupportsItemExpanding()),the methods
+     *         KItemModelBase::setExpanded(), KItemModelBase::isExpanded(),
+     *         KItemModelBase::isExpandable() and KItemModelBase::expandedParentsCount()
+     *         must be reimplemented. The view-implementation
      *         has to take care itself how to visually represent the expanded items provided
      *         by the model.
      */
-    virtual bool supportsItemExpanding() const;
+    bool supportsItemExpanding() const;
 
     /**
      * @return The rectangle of the item relative to the top/left of
@@ -202,13 +236,26 @@ public:
      */
     void scrollToItem(int index);
 
+    /**
+     * If several properties of KItemListView are changed synchronously, it is
+     * recommended to encapsulate the calls between beginTransaction() and endTransaction().
+     * This prevents unnecessary and expensive layout-calculations.
+     */
     void beginTransaction();
+
+    /**
+     * Counterpart to beginTransaction(). The layout changes will only be animated if
+     * all property changes between beginTransaction() and endTransaction() support
+     * animations.
+     */
     void endTransaction();
+
     bool isTransactionActive() const;
 
     /**
      * Turns on the header if \p show is true. Per default the
-     * header is not shown.
+     * header is not shown. Usually the header is turned on when
+     * showing a classic "table-view" to describe the shown columns.
      */
     void setHeaderShown(bool show);
     bool isHeaderShown() const;
@@ -286,6 +333,13 @@ protected:
 
     QList<KItemListWidget*> visibleItemListWidgets() const;
 
+    /**
+     * Must be called by the derived class if it supports the expanding
+     * of items.
+     * @see supportsItemExpanding()
+     */
+    void setSupportsItemExpanding(bool supportsExpanding);
+
 protected slots:
     virtual void slotItemsInserted(const KItemRangeList& itemRanges);
     virtual void slotItemsRemoved(const KItemRangeList& itemRanges);
@@ -324,6 +378,14 @@ private slots:
      */
     void triggerAutoScrolling();
 
+    /**
+     * Is invoked if the geometry of the parent-widget from a group-header has been
+     * changed. The x-position and width of the group-header gets adjusted to assure
+     * that it always spans the whole width even during temporary transitions of the
+     * parent widget.
+     */
+    void slotGeometryOfGroupHeaderParentChanged();
+
 private:
     enum LayoutAnimationHint
     {
@@ -344,22 +406,44 @@ private:
 
     void doLayout(LayoutAnimationHint hint, int changedIndex = 0, int changedCount = 0);
 
+    /**
+     * Helper method for doLayout: Returns a list of items that can be reused for the visible
+     * area. Invisible group headers get recycled. The reusable items are items that are
+     * invisible. If the animation hint is 'Animation' then items that are currently animated
+     * won't be reused. Reusing items is faster in comparison to deleting invisible
+     * items and creating a new instance for visible items.
+     */
+    QList<int> recycleInvisibleItems(int firstVisibleIndex,
+                                     int lastVisibleIndex,
+                                     LayoutAnimationHint hint);
+
+    /**
+     * Helper method for doLayout: Starts a moving-animation for the widget to the given
+     * new position. The moving-animation is only started if the new position is within
+     * the same row or column, otherwise the create-animation is used instead.
+     * @return True if the moving-animation has been applied.
+     */
+    bool moveWidget(KItemListWidget* widget, const QPointF& newPos);
+
     void emitOffsetChanges();
 
     KItemListWidget* createWidget(int index);
     void recycleWidget(KItemListWidget* widget);
+
+    /**
+     * Changes the index of the widget to \a index and assures a consistent
+     * update for m_visibleItems and m_visibleCells. The cell-information
+     * for the new index will not be updated and be initialized as empty cell.
+     */
     void setWidgetIndex(KItemListWidget* widget, int index);
 
     /**
-     * Helper method for setGeometry() and setItemSize(): Calling both methods might result
-     * in a changed number of visible items. To assure that currently invisible items can
-     * get animated from the old position to the new position prepareLayoutForIncreasedItemCount()
-     * takes care to create all item widgets that are visible with the old or the new size.
-     * @param size     Size of the layouter or the item dependent on \p sizeType.
-     * @param sizeType LayouterSize: KItemListLayouter::setSize() is used.
-     *                 ItemSize: KItemListLayouter::setItemSize() is used.
+     * Changes the index of the widget to \a index. In opposite to
+     * setWidgetIndex() the cell-information for the widget gets updated.
+     * This update gives doLayout() the chance to animate the moving
+     * of the item visually (see moveWidget()).
      */
-    void prepareLayoutForIncreasedItemCount(const QSizeF& size, SizeType sizeType);
+    void moveWidgetToIndex(KItemListWidget* widget, int index);
 
     /**
      * Helper method for prepareLayoutForIncreasedItemCount().
@@ -373,7 +457,7 @@ private:
     void updateWidgetProperties(KItemListWidget* widget, int index);
 
     /**
-     * Helper method for createWidget() and setWidgetIndex() to create or update
+     * Helper method for updateWidgetPropertes() to create or update
      * the itemlist group-header.
      */
     void updateGroupHeaderForWidget(KItemListWidget* widget);
@@ -397,6 +481,26 @@ private:
      */
     void updateVisibleGroupHeaders();
 
+    /**
+     * @return Index for the item in the list returned by KItemModelBase::groups()
+     *         that represents the group where the item with the index \a index
+     *         belongs to. -1 is returned if no groups are available.
+     */
+    int groupIndexForItem(int index) const;
+
+    /**
+     * Updates the alternateBackground-property of the widget dependent
+     * on the state of useAlternateBackgrounds() and the grouping state.
+     */
+    void updateAlternateBackgroundForWidget(KItemListWidget* widget);
+
+    /**
+     * @return True if alternate backgrounds should be used for the items.
+     *         This is the case if an empty item-size is given and if there
+     *         is more than one visible role.
+     */
+    bool useAlternateBackgrounds() const;
+
     /**
      * @return The widths of each visible role that is shown in the KItemListHeader.
      */
@@ -438,6 +542,52 @@ private:
      */
     QRectF headerBoundaries() const;
 
+    /**
+     * @return True if the number of columns or rows will be changed when applying
+     *         the new grid- and item-size. Used to determine whether an animation
+     *         should be done when applying the new layout.
+     */
+    bool changesItemGridLayout(const QSizeF& newGridSize,
+                               const QSizeF& newItemSize,
+                               const QSizeF& newItemMargin) const;
+
+    /**
+     * @param changedItemCount Number of inserted  or removed items.
+     * @return                 True if the inserting or removing of items should be animated.
+     *                         No animation should be done if the number of items is too large
+     *                         to provide a pleasant animation.
+     */
+    bool animateChangedItemCount(int changedItemCount) const;
+
+    /**
+     * @return True if a scrollbar for the given scroll-orientation is required
+     *         when using a size of \p size for the view. Calling the method is rather
+     *         expansive as a temporary relayout needs to be done.
+     */
+    bool scrollBarRequired(const QSizeF& size) const;
+
+    /**
+     * Applies the height of the group header to the layouter. The height
+     * depends on the used scroll orientation.
+     */
+    void updateGroupHeaderHeight();
+
+    /**
+     * Updates the siblings-information for all visible items that are inside
+     * the range of \p firstIndex and \p lastIndex. If firstIndex or lastIndex
+     * is smaller than 0, the siblings-information for all visible items gets
+     * updated.
+     * @see KItemListWidget::setSiblingsInformation()
+     */
+    void updateSiblingsInformation(int firstIndex = -1, int lastIndex = -1);
+
+    /**
+     * Helper method for updateExpansionIndicators().
+     * @return True if the item with the index \a index has a sibling successor
+     *         (= the item is not the last item of the current hierarchy).
+     */
+    bool hasSiblingSuccessor(int index) const;
+
     /**
      * Helper function for triggerAutoScrolling().
      * @param pos    Logical position of the mouse relative to the range.
@@ -450,10 +600,19 @@ private:
      */
     static int calculateAutoScrollingIncrement(int pos, int range, int oldInc);
 
+    /**
+     * Helper functions for changesItemCount().
+     * @return The number of items that fit into the available size by
+     *         respecting the size of the item and the margin between the items.
+     */
+    static int itemsPerSize(qreal size, qreal itemSize, qreal itemMargin);
+
 private:
     bool m_enabledSelectionToggles;
     bool m_grouped;
+    bool m_supportsItemExpanding;
     int m_activeTransactions; // Counter for beginTransaction()/endTransaction()
+    LayoutAnimationHint m_endTransactionAnimationHint;
 
     QSizeF m_itemSize;
     KItemListController* m_controller;
@@ -468,6 +627,15 @@ private:
     QHash<int, KItemListWidget*> m_visibleItems;
     QHash<KItemListWidget*, KItemListGroupHeader*> m_visibleGroups;
 
+    struct Cell
+    {
+        Cell() : column(-1), row(-1) {}
+        Cell(int c, int r) : column(c), row(r) {}
+        int column;
+        int row;
+    };
+    QHash<int, Cell> m_visibleCells;
+
     int m_scrollBarExtent;
     KItemListSizeHintResolver* m_sizeHintResolver;
     KItemListViewLayouter* m_layouter;
@@ -489,7 +657,9 @@ private:
     KItemListHeader* m_header;
     bool m_useHeaderWidths;
 
+    friend class KItemListContainer; // Accesses scrollBarRequired()
     friend class KItemListController;
+    friend class KItemListControllerTest;
 };
 
 /**
