@@ -68,6 +68,7 @@ private slots:
     void testModelConsistencyWhenInsertingItems();
     void testItemRangeConsistencyWhenInsertingItems();
     void testExpandItems();
+    void testExpandParentItems();
     void testSorting();
 
     void testExpansionLevelsCompare_data();
@@ -270,7 +271,7 @@ void KFileItemModelTest::testSetDataWithModifiedSortRole()
     if (expectMoveSignal) {
         QVERIFY(QTest::kWaitForSignal(m_model, SIGNAL(itemsMoved(KItemRange,QList<int>)), DefaultTimeout));
     }
-    
+
     QCOMPARE(m_model->data(0).value("rating").toInt(), ratingIndex0);
     QCOMPARE(m_model->data(1).value("rating").toInt(), ratingIndex1);
     QCOMPARE(m_model->data(2).value("rating").toInt(), ratingIndex2);
@@ -372,7 +373,7 @@ void KFileItemModelTest::testExpandItems()
     // yields the correct result for "a/a/1" and "a/a-1/", whis is non-trivial because they share the
     // first three characters.
     QSet<QByteArray> modelRoles = m_model->roles();
-    modelRoles << "isExpanded" << "isExpandable" << "expansionLevel";
+    modelRoles << "isExpanded" << "isExpandable" << "expandedParentsCount";
     m_model->setRoles(modelRoles);
 
     QStringList files;
@@ -478,6 +479,57 @@ void KFileItemModelTest::testExpandItems()
     QCOMPARE(m_model->expandedUrls(), allFolders);
 }
 
+void KFileItemModelTest::testExpandParentItems()
+{
+    // Create a tree structure of folders:
+    // a 1/
+    // a 1/b1/
+    // a 1/b1/c1/
+    // a2/
+    // a2/b2/
+    // a2/b2/c2/
+    // a2/b2/c2/d2/
+    QSet<QByteArray> modelRoles = m_model->roles();
+    modelRoles << "isExpanded" << "isExpandable" << "expandedParentsCount";
+    m_model->setRoles(modelRoles);
+
+    QStringList files;
+    files << "a 1/b1/c1/file.txt" << "a2/b2/c2/d2/file.txt"; // missing folders are created automatically
+    m_testDir->createFiles(files);
+
+    m_dirLister->openUrl(m_testDir->url());
+    QVERIFY(QTest::kWaitForSignal(m_model, SIGNAL(itemsInserted(KItemRangeList)), DefaultTimeout));
+
+    // So far, the model contains only "a 1/" and "a2/".
+    QCOMPARE(m_model->count(), 2);
+    QVERIFY(m_model->expandedUrls().empty());
+
+    // Expand the parents of "a2/b2/c2".
+    m_model->expandParentItems(KUrl(m_testDir->name() + "a2/b2/c2"));
+    QVERIFY(QTest::kWaitForSignal(m_model, SIGNAL(loadingCompleted()), DefaultTimeout));
+
+    // The model should now contain "a 1/", "a2/", "a2/b2/", and "a2/b2/c2/".
+    // It's important that only the parents of "a1/b1/c1" are expanded.
+    QCOMPARE(m_model->count(), 4);
+    QVERIFY(!m_model->isExpanded(0));
+    QVERIFY(m_model->isExpanded(1));
+    QVERIFY(m_model->isExpanded(2));
+    QVERIFY(!m_model->isExpanded(3));
+
+    // Expand the parents of "a 1/b1".
+    m_model->expandParentItems(KUrl(m_testDir->name() + "a 1/b1"));
+    QVERIFY(QTest::kWaitForSignal(m_model, SIGNAL(loadingCompleted()), DefaultTimeout));
+
+    // The model should now contain "a 1/", "a 1/b1/", "a2/", "a2/b2", and "a2/b2/c2/".
+    // It's important that only the parents of "a 1/b1/" and "a2/b2/c2/" are expanded.
+    QCOMPARE(m_model->count(), 5);
+    QVERIFY(m_model->isExpanded(0));
+    QVERIFY(!m_model->isExpanded(1));
+    QVERIFY(m_model->isExpanded(2));
+    QVERIFY(m_model->isExpanded(3));
+    QVERIFY(!m_model->isExpanded(4));
+}
+
 void KFileItemModelTest::testSorting()
 {
     // Create some files with different sizes and modification times to check the different sorting options
@@ -613,7 +665,7 @@ void KFileItemModelTest::testExpansionLevelsCompare()
     b.item = itemB;
     b.parent = 0;
 
-    QCOMPARE(m_model->expansionLevelsCompare(&a, &b), result);
+    QCOMPARE(m_model->expandedParentsCountCompare(&a, &b), result);
 }
 
 void KFileItemModelTest::testIndexForKeyboardSearch()
