@@ -325,7 +325,7 @@ FolderView::FolderView(QObject *parent, const QVariantList &args)
         setUrl(KUrl(args.value(0).toString()));
     }
 
-    resize(600, 400);
+    //resize(600, 400);
 
     // As we use some part of konqueror libkonq must be added to have translations
     KGlobal::locale()->insertCatalog("libkonq");
@@ -339,9 +339,10 @@ void FolderView::init()
     // Find out about icon and font settings changes
     connect(KGlobalSettings::self(), SIGNAL(kdisplayFontChanged()), SLOT(fontSettingsChanged()));
     connect(KGlobalSettings::self(), SIGNAL(iconChanged(int)), SLOT(iconSettingsChanged(int)));
+    connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)), SLOT(clickSettingsChanged(int)));
 
     // Find out about theme changes
-    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(themeChanged()));
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(plasmaThemeChanged()));
 
     // Find out about network availability changes
     connect(Solid::Networking::notifier(), SIGNAL(statusChanged(Solid::Networking::Status)),
@@ -358,6 +359,7 @@ void FolderView::init()
     m_alignToGrid         = cg.readEntry("alignToGrid", false);
     m_clickToView         = cg.readEntry("clickForFolderPreviews", true);
     m_previewPlugins      = cg.readEntry("previewPlugins", QStringList() << "imagethumbnail" << "jpegthumbnail");
+    m_customIconSize      = cg.readEntry("customIconSize", 0);
     m_sortDirsFirst       = cg.readEntry("sortDirsFirst", true);
     m_sortColumn          = cg.readEntry("sortColumn", int(KDirModel::Name));
     m_filterFiles         = cg.readEntry("filterFiles", "*");
@@ -365,6 +367,8 @@ void FolderView::init()
     m_filterFilesMimeList = cg.readEntry("mimeFilter", QStringList());
     m_blankLabel          = cg.readEntry("blankLabel", false);
     m_userSelectedShowAllFiles = m_filterType;
+    m_showSelectionMarker = KGlobalSettings::singleClick();
+
     if (isContainment()) {
         m_flow = layoutDirection() == Qt::LeftToRight ? IconView::TopToBottom : IconView::TopToBottomRightToLeft;
     } else {
@@ -466,11 +470,11 @@ void FolderView::configChanged()
         needReload = true;
     }
 
-    m_showPreviews = cg.readEntry("showPreviews", m_showPreviews);
     m_drawShadows  = cg.readEntry("drawShadows", m_drawShadows);
     m_clickToView  = cg.readEntry("clickForFolderPreviews", m_clickToView);
     m_numTextLines = cg.readEntry("numTextLines", m_numTextLines);
     m_alignToGrid  = cg.readEntry("alignToGrid", m_alignToGrid);
+
     if (QAction *action = m_actionCollection.action("auto_align")) {
         action->setChecked(m_alignToGrid);
     }
@@ -486,6 +490,21 @@ void FolderView::configChanged()
         needReload = true;
     }
 
+    const bool showPreviews = cg.readEntry("showPreviews", m_showPreviews);
+    if (showPreviews != m_showPreviews) {
+        m_showPreviews = showPreviews;
+
+        //As disabling the previews will force a rearrangement, we need to manually
+        //save and restore the icons positions
+
+        //Enable/disable the previews
+        m_previewGenerator->setPreviewShown(m_showPreviews);
+        if (m_iconView)
+            m_iconView->update(m_iconView->visibleArea());
+        if (m_listView)
+            m_listView->update(m_listView->visibleArea());
+    }
+
     m_previewPlugins = cg.readEntry("previewPlugins", m_previewPlugins);
 
     if (m_previewGenerator && m_previewPlugins != m_previewGenerator->enabledPlugins()) {
@@ -514,6 +533,11 @@ void FolderView::configChanged()
         updateSortActionsState();
     }
 
+    const int flow = static_cast<IconView::Flow>(cg.readEntry("flow", static_cast<int>(m_flow)));
+    if (flow != m_flow) {
+        m_flow = static_cast<IconView::Flow>(flow);
+    }
+
     const int filterType = cg.readEntry("filter", m_filterType);
     if (filterType != m_filterType) {
         m_filterType = filterType;
@@ -796,25 +820,7 @@ void FolderView::configAccepted()
 
     cg.writeEntry("drawShadows", uiDisplay.drawShadows->isChecked());
 
-    if (m_showPreviews != uiDisplay.showPreviews->isChecked()) {
-        cg.writeEntry("showPreviews", uiDisplay.showPreviews->isChecked());
-        //As disabling the previews will force a rearrangement, we need to manually
-        //save and restore the icons positions
-
-        QStringList iconPositionsData;
-        if (!m_showPreviews && m_iconView) {
-            //Save the icon positions
-            iconPositionsData = m_iconView->iconPositionsData();
-        }
-
-        //Enable/disable the previews
-        m_previewGenerator->setPreviewShown(m_showPreviews);
-
-        if (!m_showPreviews && m_iconView) {
-            //Restore the icon positions
-            m_iconView->setIconPositionsData(iconPositionsData);
-        }
-    }
+    cg.writeEntry("showPreviews", uiDisplay.showPreviews->isChecked());
 
     if (m_previewGenerator && m_previewPlugins != m_previewGenerator->enabledPlugins()) {
         cg.writeEntry("previewPlugins", m_previewPlugins);
@@ -967,9 +973,7 @@ void FolderView::updateListViewState()
     m_listView->setDrawShadows(m_drawShadows);
     m_listView->setIconSize(iconSize());
     m_listView->setWordWrap(m_numTextLines > 1);
-    if ( m_iconView ) {
-        m_iconView->setTextLineCount(m_numTextLines);
-    }
+    m_listView->setTextLineCount(m_numTextLines);
 }
 
 void FolderView::updateIconViewState()
@@ -986,6 +990,7 @@ void FolderView::updateIconViewState()
     m_iconView->setAlignToGrid(m_alignToGrid);
     m_iconView->setIconsMoveable(!m_iconsLocked);
     m_iconView->setClickToViewFolders(m_clickToView);
+    m_iconView->setShowSelectionMarker(m_showSelectionMarker);
 
     if (m_label) {
         m_label->setPalette(palette);
@@ -1090,10 +1095,19 @@ void FolderView::iconSettingsChanged(int group)
         m_listView->setIconSize(QSize(size, size));
         m_listView->markAreaDirty(m_listView->visibleArea());
         m_listView->update();
+
+        updateGeometry();
     }
 }
 
-void FolderView::themeChanged()
+void FolderView::clickSettingsChanged(int category)
+{
+  if (category == KGlobalSettings::SETTINGS_MOUSE && m_iconView) {
+    m_iconView->setShowSelectionMarker(KGlobalSettings::singleClick());
+  }
+}
+
+void FolderView::plasmaThemeChanged()
 {
     if (m_textColor != Qt::transparent) {
         return;
@@ -1183,8 +1197,10 @@ void FolderView::constraintsEvent(Plasma::Constraints constraints)
                 // Give the applet a sane size
                 setupIconView();
             }
+
+            resize(600, 400);
             setAspectRatioMode(Plasma::IgnoreAspectRatio);
-        } else {
+        } else if (!m_iconWidget) {
             // Clean up the icon view
             delete m_label;
             delete m_iconView;
@@ -1231,8 +1247,12 @@ void FolderView::constraintsEvent(Plasma::Constraints constraints)
             layout->addItem(m_iconWidget);
 
             setLayout(layout);
+            int iconSize = IconSize(KIconLoader::Panel);
+            setMinimumSize(QSizeF(iconSize, iconSize));
             setAspectRatioMode(Plasma::ConstrainedSquare);
         }
+
+        update();
     }
 
     if (constraints & Plasma::ScreenConstraint) {
@@ -2017,11 +2037,25 @@ void FolderView::timerEvent(QTimerEvent *event)
 QSizeF FolderView::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
 {
     if (which == Qt::PreferredSize) {
-        if (!constraint.isEmpty()) {
-            return QSizeF(600, 400).boundedTo(constraint);
-        } else {
-            return QSizeF(600, 400);
+        QSizeF size;
+
+        switch (formFactor()) {
+          case Plasma::Planar:
+          case Plasma::MediaCenter:
+              if (!constraint.isEmpty()) {
+                  size = QSizeF(600, 400).boundedTo(constraint);
+              } else {
+                  size = QSizeF(600, 400);
+              }
+              break;
+          case Plasma::Horizontal:
+          case Plasma::Vertical:
+              const int iconSize = IconSize(KIconLoader::Panel);
+              size = QSizeF(iconSize, iconSize);
+              break;
         }
+
+        return size;
     }
 
     return Containment::sizeHint(which, constraint);
