@@ -65,6 +65,7 @@ public:
     Q_PROPERTY(bool wordWrap READ wordWrap WRITE setWordWrap)
     Q_PROPERTY(bool alignToGrid READ alignToGrid WRITE setAlignToGrid)
     Q_PROPERTY(bool clickToViewFolders READ clickToViewFolders WRITE setClickToViewFolders)
+    Q_PROPERTY(bool showSelectionMarker READ showSelectionMarker WRITE setShowSelectionMarker)
     Q_PROPERTY(bool iconsMoveable READ iconsMoveable WRITE setIconsMoveable)
     Q_PROPERTY(bool customLayout READ customLayout WRITE setCustomLayout)
     Q_PROPERTY(Flow flow READ flow WRITE setFlow)
@@ -125,6 +126,9 @@ public:
     bool clickToViewFolders() const;
     void openPopup(const QModelIndex &index);
 
+    void setShowSelectionMarker(bool show);
+    bool showSelectionMarker() const;
+
 signals:
     void indexesMoved(const QModelIndexList &indexes);
     void popupViewClosed();
@@ -185,6 +189,7 @@ private slots:
     void repositionWidgetsManually();
     void closeEditor(QGraphicsWidget *editor, QAbstractItemDelegate::EndEditHint hint);
     void checkIfFolderResult(const QModelIndex &index, bool isFolder);
+    void svgChanged();
 
 private:
     void paintMessage(QPainter *painter, const QRect &rect, const QString &message,
@@ -213,7 +218,6 @@ private:
     void repaintSelectedIcons();
 
 private:
-    Plasma::FrameSvg *m_itemFrame;
     QVector<ViewItem> m_items;
     QHash<QString, QPoint> m_savedPositions;
     mutable QCache<quint64, QRegion> m_regionCache;
@@ -234,6 +238,8 @@ private:
     bool m_popupShowPreview;
     bool m_folderIsEmpty;
     bool m_clickToViewFolders;
+    bool m_showSelectionMarker;
+    bool m_drawIconShrinked;
     QPersistentModelIndex m_hoveredIndex;
     QPersistentModelIndex m_pressedIndex;
     QPersistentModelIndex m_editorIndex;
