@@ -42,7 +42,6 @@
 
 ListView::ListView(QGraphicsWidget *parent)
     : AbstractItemView(parent),
-      m_itemFrame(0),
       m_rowHeight(-1),
       m_numTextLines(2),
       m_dragInProgress(false),
@@ -52,11 +51,6 @@ ListView::ListView(QGraphicsWidget *parent)
     setAcceptDrops(true);
     setCacheMode(NoCache);
 
-    m_itemFrame = new Plasma::FrameSvg(this);
-    m_itemFrame->setImagePath("widgets/viewitem");
-    m_itemFrame->setCacheAllRenderedFrames(true);
-    m_itemFrame->setElementPrefix("normal");
-
     m_animator = new Animator(this);
 }
 
@@ -98,7 +92,7 @@ void ListView::setTextLineCount(int count)
     if (count != m_numTextLines) {
         m_numTextLines = count;
         m_rowHeight = -1;
-        markAreaDirty(visibleArea());
+        updateSizeHint();
     }
 }
 
@@ -148,6 +142,12 @@ void ListView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottom
     markAreaDirty(visualRect(topLeft) | visualRect(bottomRight));
 }
 
+void ListView::svgChanged()
+{
+    m_rowHeight = -1;
+    updateSizeHint();
+}
+
 void ListView::updateScrollBar()
 {
     if (!m_model) {
