@@ -69,6 +69,7 @@ PopupView::PopupView(const QModelIndex &index, const QPoint &pos,
       m_parentView(parentView),
       m_busyWidget(0),
       m_iconView(0),
+      m_parentViewModel(0),
       m_dirModel(0),
       m_model(0),
       m_actionCollection(this),
@@ -98,7 +99,9 @@ PopupView::PopupView(const QModelIndex &index, const QPoint &pos,
     pal.setColor(QPalette::Text, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
     setPalette(pal);
 
-    KFileItem item = static_cast<const ProxyModel*>(index.model())->itemForIndex(index);
+    m_parentViewModel = static_cast<const ProxyModel*>(index.model());
+
+    KFileItem item = m_parentViewModel->itemForIndex(index);
     if (item.isDesktopFile()) {
         KDesktopFile file(item.localPath());
         m_url = file.readUrl();
@@ -188,11 +191,12 @@ void PopupView::init()
 
     m_model = new ProxyModel(this);
     m_model->setSourceModel(m_dirModel);
-    m_model->setSortLocaleAware(true);
-    m_model->setDynamicSortFilter(true);
-    m_model->setFilterCaseSensitivity(Qt::CaseInsensitive);
-    m_model->setParseDesktopFiles(m_url.protocol() == "desktop");
-    m_model->sort(KDirModel::Name, Qt::AscendingOrder);
+    m_model->setSortLocaleAware(m_parentViewModel->isSortLocaleAware());
+    m_model->setParseDesktopFiles(m_parentViewModel->parseDesktopFiles());
+    m_model->setFilterMode(m_parentViewModel->ProxyModel::NoFilter);
+    m_model->setSortDirectoriesFirst(m_parentViewModel->sortDirectoriesFirst());
+    m_model->setDynamicSortFilter(m_parentViewModel->dynamicSortFilter());
+    m_model->sort(m_parentViewModel->sortColumn(), m_parentViewModel->sortOrder());
 
     m_delegate = new KFileItemDelegate(this);
     m_selectionModel = new QItemSelectionModel(m_model, this);
@@ -206,9 +210,11 @@ void PopupView::init()
     m_iconView->setDrawShadows(m_parentView->drawShadows());
     m_iconView->setIconSize(m_parentView->iconSize());
     m_iconView->setGridSize(m_parentView->gridSize());
+    m_iconView->setTextLineCount(m_parentView->textLineCount());
     m_iconView->setWordWrap(m_parentView->wordWrap());
     m_iconView->setIconsMoveable(false);
     m_iconView->setClickToViewFolders(false);
+    m_iconView->setShowSelectionMarker(m_parentView->showSelectionMarker());
 
     connect(m_iconView, SIGNAL(activated(QModelIndex)), SLOT(activated(QModelIndex)));
     connect(m_iconView, SIGNAL(contextMenuRequest(QWidget*,QPoint)), SLOT(contextMenuRequest(QWidget*,QPoint)));
